@@ -45,6 +45,8 @@ struct tool_mix {
     struct snd_shadow *mix_shadow;
 };
 
+/* Describes a source for mixing. */
+
 struct tool_mix_source {
     int destroy_notification_id;
     int track;
@@ -53,6 +55,8 @@ struct tool_mix_source {
     AFframecount current_pos;
 };
 
+/* The current mix source. */
+
 static struct tool_mix_source tms = { 0 };
 
 static struct cmd_value *
@@ -85,25 +89,6 @@ tool_mix_release_source() {
     }
 }
 
-void
-tool_mix_acquire_source(struct clip *clip,
-                        int track,
-                        AFframecount offset) {
-    tool_mix_release_source();
-
-    tms.clip = clip;
-    tms.track = track;
-    tms.offset = offset;
-    tms.current_pos = -1;
-    
-    tms.destroy_notification_id = 
-        msg_subscribe(clip->msg,
-                      "clip::destroy",
-                      "tool-mix::source-destroyed", 
-                      NULL);
-
-}
-
 void
 tool_mix_update_source(AFframecount current_pos) {
     struct tool_mix_source *tms = tool_mix_get_source();
@@ -112,6 +97,38 @@ tool_mix_update_source(AFframecount current_pos) {
         return;
 
     tms->current_pos = current_pos;
+
+}
+
+void
+tool_mix_acquire_source(shell *shl,
+                        int track,
+                        AFframecount offset) {
+    char s[20];
+    tool_mix_release_source();
+
+    tms.clip = shl->clip;
+    tms.track = track;
+    tms.offset = offset;
+    tms.current_pos = -1;
+
+    if(offset >= 0) {
+        grid_format(&shl->grid,
+                    offset,
+                    s,
+                    sizeof s,
+                    GRID_FORMAT_LONG);
+
+        view_set_transient(shl->view, MSG_INFO, "Source at %s", s);
+        view_redraw(shl->view);
+    }
+
+    tms.destroy_notification_id = 
+        msg_subscribe(shl->clip->msg,
+                      "clip::destroy",
+                      "tool-mix::source-destroyed", 
+                      NULL);
+
 }
 
 
@@ -395,21 +412,19 @@ tool_mix_commit(struct tool *tool) {
 
     tms = tool_mix_get_source();
 
+    DEBUG("tms: %p, tm->pending: %d\n", tms, tm->pending);
+
     if(!tm->pending)
         return cmd_new_void_val();
 
     if(!tms) 
         return cmd_new_void_val();
 
+    DEBUG("committing...\n");
+
     tool_mix_update_source(-1);
 
     history_begin(shl->history, "Mix Tool");
-    history_remember(shl->history,
-                     CMD_NEW("delete-snd",
-                             cmd_new_shellp_val(shl),
-                             cmd_new_int_val(tm->mix_shadow->map),
-                             cmd_new_long_val(tm->mix_shadow->offset),
-                             cmd_new_long_val(tm->mix_shadow->count)));
     history_remember(shl->history,
                      CMD_NEW("insert-snd",
                              cmd_new_shellp_val(shl),
@@ -418,6 +433,12 @@ tool_mix_commit(struct tool *tool) {
                                                         cmd_sndp_dtor),
                              cmd_new_int_val(tm->mix_shadow->map),
                              cmd_new_long_val(tm->mix_shadow->offset)));
+    history_remember(shl->history,
+                     CMD_NEW("delete-snd",
+                             cmd_new_shellp_val(shl),
+                             cmd_new_int_val(tm->mix_shadow->map),
+                             cmd_new_long_val(tm->mix_shadow->offset),
+                             cmd_new_long_val(tm->mix_shadow->count)));
     history_commit(shl->history);
 
     snd_dump(tm->mix_shadow->dst);
@@ -445,7 +466,7 @@ tool_mix_button_press(struct tool *tool,
         return cmd_new_void_val();
 
     if(event->state & GDK_CONTROL_MASK) {
-        tool_mix_acquire_source(shl->clip, track, x);
+        tool_mix_acquire_source(shl, track, x);
         gtk_widget_queue_draw(view_get_widget(shl->view, "wavecanvas"));
         return cmd_new_void_val();
     }
@@ -475,18 +496,18 @@ tool_mix_motion(struct tool *tool,
 
     AFframecount x = PIXEL_TO_FRAME(shl->view, event->x), x1, x2;
 
-    if(track != tm->mix_track)
-        return cmd_new_void_val();
-
     if(!(event->state & GDK_BUTTON1_MASK)) 
         return cmd_new_void_val();
 
     if(event->state & GDK_CONTROL_MASK) {
-        tool_mix_acquire_source(shl->clip, track, x);
+        tool_mix_acquire_source(shl, track, x);
         gtk_widget_queue_draw(view_get_widget(shl->view, "wavecanvas"));
         return cmd_new_void_val();
     }
 
+    if(track != tm->mix_track)
+        return cmd_new_void_val();
+
     tms = tool_mix_get_source();
 
     if(!tms) 
@@ -549,7 +570,8 @@ tool_mix_button_release(struct tool *tool,
     AFframecount x = PIXEL_TO_FRAME(shl->view, event->x);
 
     if(event->state & GDK_CONTROL_MASK) {
-        tool_mix_acquire_source(shl->clip, track, x);
+        tool_mix_acquire_source(shl, track, x);
+        view_clear_transient(shl->view);
         gtk_widget_queue_draw(view_get_widget(shl->view, "wavecanvas"));
         return cmd_new_void_val();
     }
@@ -614,7 +636,7 @@ tool_mix_init(int id) {
 
         { "tool-mix::source-destroyed", "Clip destroy event.",
           tool_mix_source_destroyed, CMD_VOID_T, 
-          cmd_new_paramdecl(1, CMD_clipp_T) },
+          cmd_new_paramdecl(2, CMD_clipp_T, CMD_voidp_T) },
 
     };
 
