@@ -842,7 +842,7 @@ player_thread(void *ptr) {
 
  cleanup_start_failed:    
     pd->stop(p);
- cleanup:
+ cleanup: ;
     pthread_cleanup_pop(1);
     return NULL;
 }
