@@ -60,9 +60,11 @@ snd_shadow_new(snd *src,
 
     if(!shadow->dst) {
         mem_free(shadow);
+        FAIL("could not copy src: map: %d, offset: %ld, count: %ld (src length: %ld)\n", map, offset, count, snd_frame_count(src, MAP_ALL));
         return NULL;
     }
 
+    snd_set_name(shadow->dst, "shadow");
     snd_dump(shadow->dst);
     shadow->src = src;
     shadow->offset = shadow->pivot = offset;
