@@ -1,7 +1,7 @@
 
 /*
  * GNUsound - a sound editor for GNOME.
- * Copyright (C) 2002-2004  Pascal Haakmat <a.haakmat@chello.nl>
+ * Copyright (C) 2002-2005  Pascal Haakmat <a.haakmat@chello.nl>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,11 +47,7 @@
 void
 view_disconnect_from_clip(struct view *view,
                           struct clip *clip) {
-    int i;
-    
-    for(i = 0; i < view->msg_count; i++) 
-        msg_unsubscribe(clip->msg,
-                        view->msg_ids[i]);
+    msg_unsubscribe_by_data(clip->msg, view);
 }
 
 void
@@ -61,30 +57,44 @@ view_connect_to_clip(struct view *view,
     /* The message handlers ("view-track-inserted" etc) are defined in
        cmd_view.c. */
 
-    view->msg_count = 0;
-    view->msg_ids[view->msg_count++] = 
-        msg_subscribe(clip->msg, 
-                      "clip::track-inserted",
-                      "view-track-inserted", 
-                      view);
-    view->msg_ids[view->msg_count++] = 
-        msg_subscribe(clip->msg,
-                      "clip::track-deleted",
-                      "view-track-deleted",
-                      view);
-    view->msg_ids[view->msg_count++] = 
-        msg_subscribe(clip->msg,
-                      "clip::track-moved",
-                      "view-track-moved",
-                      view);
-    view->msg_ids[view->msg_count++] = 
-        msg_subscribe(clip->msg,
-                      "clip::snd-changed",
-                      "view-clip-snd-changed",
-                      view);
+    msg_subscribe(clip->msg, 
+                  "clip::track-inserted",
+                  "view-track-inserted", 
+                  view);
+    msg_subscribe(clip->msg,
+                  "clip::track-deleted",
+                  "view-track-deleted",
+                  view);
+    msg_subscribe(clip->msg,
+                  "clip::track-moved",
+                  "view-track-moved",
+                  view);
+    msg_subscribe(clip->msg,
+                  "clip::snd-changed",
+                  "view-clip-snd-changed",
+                  view);
 
 }
 
+void
+view_disconnect_from_history(struct view *view,
+                             struct history *history) {
+    msg_unsubscribe_by_data(history->msg, view);
+}
+
+void
+view_connect_to_history(struct view *view,
+                        struct history *history) {
+
+    /* The message handlers ("view-track-inserted" etc) are defined in
+       cmd_view.c. */
+
+    msg_subscribe(history->msg, 
+                  "history::committed",
+                  "view-history-changed", 
+                  view);
+}
+
 void
 view_set_default_track_draw_hooks(struct view *v,
                                   int track) {
@@ -1183,6 +1193,7 @@ void
 view_destroy(struct view *view) {
     GList *dialogs = NULL, *l;
 
+    view_disconnect_from_history(view, view->shl->history);
     view_disconnect_from_clip(view, view->shl->clip);
 
     if(view->wavepixmap)
@@ -1616,6 +1627,7 @@ view_new(shell *shl) {
     /* Connect messages. */
 
     view_connect_to_clip(v, v->shl->clip);
+    view_connect_to_history(v, v->shl->history);
 
     /* Perform final fixups. */
     
@@ -1623,6 +1635,5 @@ view_new(shell *shl) {
 
     view_reset_status(v);
 
-
     return v;
 }
