@@ -97,7 +97,7 @@ view_about_activate(GtkWidget *w,
           "GNUsound comes with ABSOLUTELY NO WARRANTY.\n"
           "This is free software, and you are welcome to redistribute it\n"
           "under certain conditions.\n \n"
-          "http://awacs.dhs.org/software/gnusound");
+          "http://www.gnu.org/software/gnusound");
           
 #ifdef HAVE_GNOME2
     static GdkPixbuf *logo_pixbuf = NULL;
